package main

import (
	"bufio"
	"bytes"
	"fmt"
	"io"
	"log"
	"net"
	"net/http"
	"os"
	"strings"
	"time"

	"github.com/grandcat/zeroconf"
	"golang.org/x/net/context"
)

type Bridge struct {
	IP   net.IP
	Name string
}

type RegisteredApplication struct {
	bridge          Bridge
	applicationName string
	deviceName      string
	token           string
}

func discoverHueBridge() ([]Bridge, error) {
	// Create a new resolver
	resolver, err := zeroconf.NewResolver(nil)
	if err != nil {
		log.Fatalf("Failed to initialize resolver: %v", err)
	}

	// Instantiate a new bridges slice
	bridges := make([]Bridge, 0)

	// Set up a context with a timeout
	ctx, cancel := context.WithTimeout(context.Background(), time.Second*3)
	defer cancel()

	// Channel to recive the results
	entries := make(chan *zeroconf.ServiceEntry)

	// Start the lookup for the Hue Bridge
	go func() {
		err = resolver.Browse(ctx, "_hue._tcp", "local.", entries)
		if err != nil {
			log.Fatalf("Failed to browse for Hue Bridge: %v", err)
		}
	}()

	// Process the results
	go func() {
		for entry := range entries {
			bridges = append(bridges, Bridge{IP: entry.AddrIPv4[0], Name: entry.HostName})
		}
	}()

	// Wait for the context to expire
	<-ctx.Done()
	fmt.Println("mDNS query finished")
	fmt.Printf("Number of devices found: %d\n", len(bridges))
	for _, device := range bridges {
		fmt.Printf("Device: %s\n", device.Name)
		fmt.Printf("IP Address: %s\n", device.IP)
	}
	return bridges, nil
}

func registerApplication(bridge Bridge) RegisteredApplication {
	var applicationName string
	var deviceName string
	fmt.Printf("Enter name of your application: ")
	fmt.Scan(&applicationName)
	fmt.Printf("Enter name of your device: ")
	fmt.Scan(&deviceName)
	resp, err := http.Post(
		"http://"+bridge.IP.String()+"/api",
		"application/json",
		bytes.NewBuffer([]byte(`{"devicetype":"`+applicationName+`#`+deviceName+`"}`)))
	if err != nil {
		log.Fatal(err)
	}
	body, err := io.ReadAll(resp.Body)
	defer resp.Body.Close()
	if err != nil {
		log.Fatal(err)
	}
	// if body has link button not pressed, tell user to press it then send the request again to register the application
	if strings.Contains(string(body), "link button not pressed") {
		fmt.Println("Link button not pressed. Press link button on bridge and press anter to continue.")
		bufio.NewReader(os.Stdin).ReadBytes('\n')
		resp.Body.Close()
		http.Post(
			"http://"+bridge.IP.String()+"/api",
			"application/json",
			bytes.NewBuffer([]byte(`{"devicetype":"`+applicationName+`#`+deviceName+`"}`)))
		body, err = io.ReadAll(resp.Body)
		if err != nil {
			log.Fatal(err)
		}
	}
	registeredApplication := RegisteredApplication{
		bridge:          bridge,
		applicationName: applicationName,
		deviceName:      deviceName,
		token:           string(body)}
	resp.Body.Close()
	return registeredApplication
}

func main() {
	bridges, err := discoverHueBridge()
	if err != nil {
		log.Fatal(err)
	}
	registeredApplications := make([]RegisteredApplication, 0)
	for _, bridge := range bridges {
		newApplication := registerApplication(bridge)
		registeredApplications = append(registeredApplications, newApplication)
	}
	fmt.Println(registeredApplications)
}
